/* -*- Mode: C; indent-tabs-mode: t; c-basic-offset: 4; tab-width: 4 -*- */
/*
 * Copyright (C) 2010, 2011 Red Hat, Inc.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA  02110-1301  USA
 *
 * Author: Jan Görig <jgorig@redhat.com>
 * 	   Adam Tkac <atkac@redhat.com>
 */

#ifndef LIBNMSERVER_H
#define LIBNMSERVER_H

#include <stdbool.h>
#include <dbus/dbus.h>
#include <pthread.h>
#include <netinet/in.h>

/*
 * This file contains public libnmserver API
 */

#define LIBNMSERVER_MAJORVER	0
#define LIBNMSERVER_MINORVER	0
#define LIBNMSERVER_PATCHVER	92

/*
 * Return codes returned by libnmserver functions
 */
typedef enum {
	NMSERVER_R_OK = 0,	/* Success */
	NMSERVER_R_FAILURE,	/* Generic failure */
	NMSERVER_R_MEMORY,	/* Not enough memory */
	NMSERVER_R_CONN,	/* Failed to estabilish connection to D-Bus */
	NMSERVER_R_NORESP,	/* NetworkManager doesn't respond */
	NMSERVER_R_BADMSG,	/* Got unexpected message from NetworkManager */
	NMSERVER_R_MATCHFAIL,	/* Failed to create match rule on D-Bus */
	NMSERVER_R_PIPE		/* Failed create pipe() based control channel */
} nmserver_result_t;

/*
 * Register this callback to track changes of nameservers. Callback returns
 * zero-terminated array of the actual valid nameservers.
 *
 * This is the example of the "addrs" parser:
 * 
 * in_addr_t *ptr;
 * for (ptr = addrs; *ptr != NULL; ptr++) {
 * 	do_whatever(*ptr);
 * }
 */
typedef void (*nmserver_nscallback_t) (in_addr_t *addrs, void *data);

/*
 * Asynchronous error reports from libnmserver. Check result to get more
 * information what happened.
 */
typedef void (*nmserver_errcallback_t) (nmserver_result_t);

/* libnmserver control handle */
typedef struct nmserver_control nmserver_control_t;

/*
 * Create libnmserver control handle. "data" is passed as nscallback parameter.
 */
nmserver_result_t nmserver_control_create(nmserver_control_t **pcontrol,
					  void *data);

/*
 * Destroy libnmserver cotrol handle
 */
void nmserver_control_destroy(nmserver_control_t **pcontrol);

/* Register callback to track changes of nameservers */
void nmserver_nscallback_register(nmserver_control_t *control,
				  nmserver_nscallback_t callback);

/* Register error callback */
void nmserver_errcallback_register(nmserver_control_t *control,
				   nmserver_errcallback_t callback);

/*
 * Run library in blocking mode. This call never returns.
 */
void nmserver_run(nmserver_control_t *control);

/*
 * Runs library in the new thread. All changes/errors are reported
 * asynchronously.
 */
nmserver_result_t nmserver_threaded_run(nmserver_control_t *control);

#endif
